@@ -13,7 +13,7 @@ __author__ = 'Steven McGrath'
 __email__ = 'steve@chigeek.com'
 
 class BaskitCLI(cmd.Cmd):
-    prompt = 'baskit_ng>'
+    prompt = 'baskit> '
     server = None
     
     def __init__(self):
@@ -25,19 +25,80 @@ class BaskitCLI(cmd.Cmd):
         for item in [self.server.env,
                      os.path.join(self.server.env, 'archive', 'backups'),
                      os.path.join(self.server.env, 'archive', 'snaps'),
+                     os.path.join(self.server.env, 'persistent'),
                      os.path.join(self.server.env, 'env')]:
             if not os.path.exists(item):
                 os.makedirs(item)
+
+
+    def do_help(self, s):
+        if s == '': self.onecmd('help help')
+        else:
+            cmd.Cmd.do_help(self, s) 
+
     
-    def help_help(self, s):
-        print 'help [COMMAND]'
+
+    def help_help(self):
+        print '''Welcome to the Baskit Minecraft Server Manager!
+        
+        Baskit is designed to be a minimalistic, yet powerful server management
+        system.  Baskit uses GNU-Screen at the heart of the application and
+        simply wrap around that.  This means that you can perform in-place
+        upgrades of Baskit, or even remove it completely without worry of it
+        impacting the running server. There are a lot of functions available to 
+        help assist you in managing the server as well, below is a high-level
+        list of what is available.  For further detail (including avalable
+        options) for any command, type help [COMMAND].
+
+        start                       Starts the server.
+        
+        stop                        Stops the server.
+
+        restart                     Restarts the server.
+
+        server                      Returns the running state & server binary
+                                    information that we have on-hand about
+                                    the running server.
+
+        cmd [COMMAND]               Sends the command specified to the server.
+                                    There is no limit as to what can be sent
+                                    here, and it is quite easy to script
+                                    commands into the server with this.
+
+        players                     Returns the list of players currently logged
+                                    into the server.
+
+        console                     Will drop you directly into the server
+                                    console.  From here you are directly
+                                    interacting with the server.  To detach from
+                                    the server console, hit CNTRL+A,D to exit.
+
+        update                      Allows you to update the server binary based
+                                    on the conditions you had specified.  It's 
+                                    highly recommended you run help update to
+                                    get some more detail.
+
+        backup                      All backup related functions are housed
+                                    within the backup command.  Running "backup"
+                                    will present it's help.
+
+        snapshot                    All snapshot related functions are housed
+                                    within the snapshot command.  Running
+                                    "snapshot" will present it's help.
+
+        sync                        Handles syncing to/from ramdisk and
+                                    persistent storage if ramdisk support is
+                                    enabled (disabled by default)
+        '''
     
+
     def do_exit(self, s):
         '''exit
         Exits the interactive CLI
         '''
         return True
     
+
     def do_cmd(self, s):
         '''cmd [COMMAND]
         Sends the specified command to the Minecraft server console
@@ -45,6 +106,7 @@ class BaskitCLI(cmd.Cmd):
         self.server.command(s)
         print 'Sent to console: %s' % s
     
+
     def do_players(self, s):
         '''players
         Retuns a list of the currently connected players
@@ -52,16 +114,18 @@ class BaskitCLI(cmd.Cmd):
         if not self.server.running():
             print 'Server must be running for this command to function'
         else:
-            print 'Players Online: %s' % self.server.players()
+            print 'Players Online: %s' % ', '.join(self.server.players())
     
+
     def do_start(self, s):
         '''start
         Starts the Minecraft service instance.
         '''
         self.server.start()
         time.sleep(0.1)
-        self.do_status(s)
+        self.do_server(s)
     
+
     def do_stop(self, s):
         '''stop [OPTIONS]
         Stops the Minecraft service instance.
@@ -111,6 +175,7 @@ class BaskitCLI(cmd.Cmd):
         self.server.stop()
         print 'Minecraft service has been stopped'
     
+
     def do_restart(self, s):
         '''restart
         Convenience function to restart the server.  No options are provided.
@@ -118,22 +183,28 @@ class BaskitCLI(cmd.Cmd):
         self.server.stop()
         self.server.start()
     
-    def do_status(self, s):
-        '''status
+
+    def do_server(self, s):
+        '''server
         Returns the current server status as well as some known configuration
         information about the server.
         '''
-        if self.server.running():
-            print 'Minecraft service is running'
-        else:
-            print 'Minecraft service is stopped'
-        print '\nServer Binary Information\n-------------------------'
+        d1 = {True: 'running', False: 'stopped'}
+        print 'Server Binary Information\n-------------------------'
+        print 'Status : %s' % d1[self.server.running()]
         print 'Type   : %s' % self.server.server_type
         print 'Branch : %s' % self.server.server_branch
         print 'Build  : %s' % self.server.server_build
-        print 'Worlds : %s' % ', '.join([world.name for world in\
-                                         self.server.worlds])
-    
+        print 'Worlds : %s' % ', '.join(self.server.worlds)
+
+
+    def do_status(self, s):
+        '''status
+        Returns information about the health of the server.
+        '''
+        pass
+
+
     def do_console(self, s):
         '''console
         Opens the Minecraft service console.  To exit the console 
@@ -141,6 +212,7 @@ class BaskitCLI(cmd.Cmd):
         '''
         self.server.console()
     
+
     def do_update(self, s):
         '''update [server_branch/build] [server_type]
         Updates the minecraft server binary to the specified information.
@@ -161,8 +233,9 @@ class BaskitCLI(cmd.Cmd):
         if len(cmd_in) > 1:
             bin_type = cmd_in[1]
         self.server.update(branch, bin_type)
-        self.do_status('')
+        self.do_server('')
     
+
     def do_backup(self, s):
         '''backup
         Handles all backup functions including management of backups.
@@ -174,8 +247,9 @@ class BaskitCLI(cmd.Cmd):
         if len(s) > 1:
             Backup(self.server).onecmd(s)
         else:
-            Backup(self.server).cmdloop(msg)
+            Backup(self.server).onecmd('help')
     
+
     def do_snapshot(self, s):
         '''snapshot
         Handles all snapshot functions including management of all snapshots.
@@ -187,30 +261,69 @@ class BaskitCLI(cmd.Cmd):
         if len(s) > 1:
             Snapshot(self.server).onecmd(s)
         else:
-            Snapshot(self.server).cmdloop(msg)
-    
-    def do_world(self, s):
-        '''world
-        Handles all world management functions for the server.  For more
-        information run:
-        
-        world help
+            Snapshot(self.server).onecmd('help')
+
+
+    def do_sync(self, s):
+        '''sync [destination] [world1, [world2, world3]]
+        Handles synchronization between the persistent and ramdisk storage.
+        By default it will sync all of the configured worlds to the destination
+        data store specified (e.g. specifying ramdisk will sync from persistent
+        to ramdisk).  Optionally you can specify specific worlds that you would
+        like to sync.  These are comma-deliited.
+
+        destination                     This is the destination of the sync.
+                                        Can either be ramdisk or persistent.
         '''
-        msg = 'type exit to return the main console'
-        if len(s) > 1:
-            WorldCLI(self.server).onecmd(s)
-        else:
-            WorldCLI(self.server).cmdloop(msg)
+        opts = s.split()
+        worlds = None
+        if len(opts) > 1:
+            worlds = [a.strip() for a in ''.join(opts[1:]).split(',')]
+        if opts[0] == 'ramdisk': self.server.prsync(worlds)
+        if opts[0] == 'persistent': self.server.rpsync(worlds)
+
 
 class Backup(cmd.Cmd):
-    prompt = 'baskit_ng [backup]>'
     server = None
-    
+
     def __init__(self, server):
         cmd.Cmd.__init__(self)
         self.server = server
         self.backup_path = os.path.join(self.server.env, 'archive', 'backups')
+
+
+    def do_help(self, s):
+        if s == '': self.onecmd('help help')
+        else:
+            cmd.Cmd.do_help(self, s) 
+
+
+    def help_help(self):
+        print '''Backup Management Functions 
+
+        Backups consist of only world data.  Worlds are backed up individually
+        ands stored on disk as zip files.
+
+        new [WORLD] [BACKUPNAME]        Creates a new backup of the world
+                                        specified.  Optionally you can name the 
+                                        backup as well.
+
+        remove [name|age] [VAL]         Removes backups based on the condition
+                                        sent.  If the condition is name, then
+                                        it will remove the backup specified in
+                                        the value.  If the condition is set to
+                                        age, then it will remove backups older
+                                        than the number of days specified in the
+                                        value.
+
+        restore [BACKUPNAME] [WORLD]    Restored the backup specified to the
+                                        world specified.
+
+        list                            Returns a list of the backups currently
+                                        in the pool.
+        '''
     
+
     def do_new(self, s):
         '''new [world_name] [backup_name]
         Creates a new world backup from the world specified, optionally a
@@ -231,6 +344,7 @@ class Backup(cmd.Cmd):
             print ('Not a configured world.  If this world exists,\n',
                    ' please add it to the configuration with world add')
     
+
     def do_remove(self, s):
         '''remove [name|age] [value]
         Removes backups
@@ -257,6 +371,7 @@ class Backup(cmd.Cmd):
                     os.remove(filename)
                     print 'Backup %s Deleted' % filename[:-4]
     
+
     def do_restore(self, s):
         '''restore [backup name] [world name]
         Restores the defined backup to the defined world name.
@@ -281,15 +396,42 @@ class Backup(cmd.Cmd):
         for filename in os.listdir(self.backup_path):
             print filename[:-4]
 
+
 class Snapshot(cmd.Cmd):
-    prompt = 'baskit_ng [snapshot]>'
-    server = None
-    
+    server = None    
+
     def __init__(self, server):
         cmd.Cmd.__init__(self)
         self.server = server
         self.backup_path = os.path.join(self.server.env, 'archive', 'snaps')
+
+
+    def do_help(self, s):
+        if s == '': self.onecmd('help help')
+        else:
+            cmd.Cmd.do_help(self, s) 
+
+
+    def help_help(self):
+        print '''Snapshot Management Functions 
+
+        Snapshots are point-in-time backups of the running binaries, plugins,
+        configurations, and anything else that isnt world data.  These are very
+        useful to run before performing either a server binary upgrade or
+        upgrading some plugins that the server uses.
+
+        new [SNAPNAME]          Generates a new snapshot.  The name is optional.
+
+        remove [SNAPNAME]       Removed the specified snapshot from the pool.
+
+        restore [SNAPNAME]      Restores a snapshot to disk.  The server must
+                                be shut down prior to restoring a snapshot as
+                                it overwites all binaries, plugins, and configs.
+
+        list                    Returns a list of the snapshots in the pool.
+        '''
     
+
     def do_new(self, s):
         '''new [snapshot_name]
         Creates a new snapshot with optionally a specified name.
@@ -302,6 +444,7 @@ class Snapshot(cmd.Cmd):
         self.server.env_snapshot(name)
         print 'Snapshot completed.'
     
+
     def do_remove(self, s):
         '''remove [name|age] [value]
         Removes snapshots
@@ -328,6 +471,7 @@ class Snapshot(cmd.Cmd):
                     os.remove(filename)
                     print 'Snapshot %s Deleted' % filename[:-4]
     
+
     def do_restore(self, s):
         '''restore [snapshot name]
         Restores the defined snapshot.
@@ -343,6 +487,7 @@ class Snapshot(cmd.Cmd):
         else:
             print 'Snapshot does not exist.'
     
+
     def do_list(self, s):
         '''list
         Returns the list of snapshots available.
@@ -351,31 +496,6 @@ class Snapshot(cmd.Cmd):
             print filename[:-4]
 
 
-class WorldCLI(cmd.Cmd):
-    prompt = 'baskit_ng [snapshot]>'
-    server = None
-    
-    def __init__(self, server):
-        cmd.Cmd.__init__(self)
-        self.server = server
-    
-    def do_add(self, s):
-        '''add [world_name]
-        Adds the world name specified to the baskit coonfiguration.  This is
-        needed in order for baskit to be able to backup and restore world
-        backups & manage ramdisks.
-        '''
-        self.server.world_add(s)
-    
-    def do_rm(self, s):
-        '''rm [world name]
-        Removes the world name specified from the list of worlds that the
-        current server configuration can see.  Configuration artifacts may
-        still exist, however are no longer needed in the configuration file.
-        '''
-        self.server.world_rm(s)
-
-
 slogans = [
     'Managing Minecraft so you dont have to.',
     'Helping minecraft admins since 2011.',
@@ -407,7 +527,7 @@ Please report any bugs or problems to the #bukget IRC channel on irc.esper.net
 as well as filing them on out github page.
 
 GitHub Repository: https://github.com/SteveMcGrath/baskit
-Informational Page: http://bukget.org/baskit
+Informational Page: http://bukget.org/pages/baskit.html
 
 Baskit: %s
 ''' % (__version__, 
