@@ -6,12 +6,11 @@ import datetime
 from zipfile import ZipFile
 from commands import getoutput as run
 from ConfigParser import ConfigParser
-
-from world import World
 import mc
 import screen
 import config
 
+
 def compress_folder(zip_filename, location, from_loc=None, excludes=[]):
     zfile = ZipFile(zip_filename, 'a')
     for dirpath, dirs, files in os.walk(location):
@@ -24,6 +23,7 @@ def compress_folder(zip_filename, location, from_loc=None, excludes=[]):
             zfile.write(fn, arc_name)
     zfile.close()
 
+
 class Server(object):
     name = 'default'
     java_args = ''
@@ -35,18 +35,21 @@ class Server(object):
     worlds = []
     _config_file = 'baskit.conf'
     
+
     def __init__(self):
         '''Baskit server initialization
         '''
         self._config_file = config.get_config_file()
         self.get_config()
     
+
     def running(self):
         '''running
         Returns True/False whether the server is running or not.
         '''
         return screen.exists('mc_%s' % self.name)
     
+
     def get_config(self, config_file=None):
         '''get_config
         Fetches the configuration for the server and adjusts the variables
@@ -72,6 +75,9 @@ class Server(object):
         self.server_build = conf.get(section, 'server_build')
         self.min_mem = conf.get(section, 'min_mem')
         self.max_mem = conf.get(section, 'max_mem')
+        self.logging = conf.getboolean(section, 'screen_log')
+        self.termout = conf.getboolean(section, 'terminal_out')
+        self.ramdisk = conf.getboolean(section, 'ramdisk')
         
         # Linking in the worlds that we are aware of to the server
         # configuration.  This is normally entirely optional unless you would
@@ -79,8 +85,9 @@ class Server(object):
         # use ramdisks.
         for world_name in conf.get(section, 'worlds').split(','):
             world_name = world_name.strip()
-            self.worlds.append(World(world_name))
+            self.worlds.append(world_name)
     
+
     def set_config(self):
         '''set_config
         Commits the current configuration of the server object to the config
@@ -104,37 +111,32 @@ class Server(object):
         conf.set(section, 'server_type', self.server_type)
         conf.set(section, 'server_branch', self.server_branch)
         conf.set(section, 'server_build', self.server_build)
-        conf.set(section, 'min_memory', self.min_mem)
-        conf.set(section, 'max_memory', self.max_mem)
+        conf.set(section, 'min_mem', self.min_mem)
+        conf.set(section, 'max_mem', self.max_mem)
         conf.set(section, 'java_args', self.java_args)
-        
-        # Now to get the list of world names that are configured with this
-        # server and add them to the 'worlds' option in the configuration
-        # stanza.
-        wnames = []
-        for world in self.worlds:
-            wnames.append(world.name)
-        conf.set(section, 'worlds', ', '.join(wnames))
+        conf.set(section, 'screen_log', self.logging)
+        conf.set(section, 'terminal_out', self.termout)
+        conf.set(section, 'ramdisk', self.ramdisk)
+        conf.set(section, 'worlds', ', '.join(self.worlds))
         
         # Lastly, we need to write the config file to disk.
         with open(self._config_file, 'wb') as configfile:
             conf.write(configfile)
         
     
-    def command(self, command, restring=None):
+    def command(self, command, *restring):
         '''command [command]
         Sends the specified command to the server console and will optionally
         wait for a given regex to return a match from the server.log before
         returning.
         '''
         
-        if restring is not None:
+        if len(restring) > 0:
             # If a regex was specified for interactivity, then we will need
             # to open the server.log file and seek to the end of the file
             # before we actually do anything.  Once we send the command that
             # was specified we will use the end of the file as a starting
             # point for the parsing.
-            relog = re.compile(restring)
             logfile = open(os.path.join(self.env, 'env', 'server.log'), 'r')
             size = os.stat(os.path.join(self.env, 'env', 'server.log'))[6]
             logfile.seek(size)
@@ -142,39 +144,44 @@ class Server(object):
         # Sending the command to the screen session
         screen.send('mc_%s' % self.name, command)
         
-        if restring is not None:
+        if len(restring) > 0:
             # Now we will start parsing through all of the log data that the
             # server is returning, using the now old EOF as a starting point.
             # We will run the regex that we had compiled to the relog variable
             # on every line until we get a match, then return those matches.
-            found = False
-            while not found:
+            found = 0
+            relog = re.compile(restring[found])
+            while found < len(restring):
                 where = logfile.tell()
                 line = logfile.readline()
                 if not line:
                     time.sleep(0.1)
                     logfile.seek(where)
                 else:
-                    if len(relog.findall(line)) > 0:
-                        found = True
+                    data = re.compile(restring[found]).findall(line)
+                    if len(data) > 0:
+                        found += 1
             logfile.close()
-            return relog.findall(line)
+            return data
         
         # If there was no regex specified, then there is nothing to return.
         return None
     
+
     def msg(self, message):
         '''msg [Message]
         Sends the specified message to all players.
         '''
         self.command('say %s' % message)
     
+
     def console(self):
         '''console
         shortcut to open the screen session directly.
         '''
         screen.console('mc_%s' % self.name)
     
+
     def start(self):
         '''start
         Starts the Minecraft server and runs any per-world initialization.
@@ -188,8 +195,7 @@ class Server(object):
             # Next we will run the per-world initialization.  Normally this
             # shouldn't do anything, however if the worlds are stored on
             # ramdisks then there will be some useful setup stuff happening.
-            for world in self.worlds:
-                world.init()
+            self.prsync()
             
             # Now just to setup all the variables, determine which java we
             # will be running, and to build the bash command that we will send
@@ -203,8 +209,9 @@ class Server(object):
                                                                self.max_mem, 
                                                                self.binary)
             # And here we go, startup time!
-            screen.new('mc_%s' % self.name, startup)
+            screen.new('mc_%s' % self.name, startup, self.logging)
     
+
     def stop(self):
         '''stop
         Tells the minecraft server to stop.
@@ -220,18 +227,24 @@ class Server(object):
         # are ramdisk worlds, then we will be performing some actions here in
         # order to properly cleanup and make sure all the ramdisk data is
         # synced.
-        for world in self.worlds:
-            world.cleanup()
+        self.rpsync()
     
+
     def players(self):
         '''players
         Returns the list of currently commected players
         '''
-        repl = re.compile(r' ([^,]*)')
-        line = self.command('list', r'players online:(.*)')[0].strip('\n')
-        players = repl.findall(line)
+        strings = {
+            'vanilla': [r'players online:', r'INFO\](.*)$'],
+            'bukkit': [r'players online:', r'(.*)$'],
+        }
+        line = self.command('list', *strings[self.server_type])[0]
+        line = line.strip('\n').strip('\x1b[m')
+        players = [a.strip() for a in line.split(',')]
+        if players[0] == '': players = []
         return players
     
+
     def update(self, build_type='stable', bin_type=None):
         if self.running():
             return False
@@ -257,6 +270,7 @@ class Server(object):
         
         return True # We have to return something so that we know it worked ;)
         
+
     def world_backup(self, world_name, backup_name=None):
         '''backup [world name], [backup_name]
         Runs the backup procedure for the specified world.
@@ -282,6 +296,7 @@ class Server(object):
                                 os.path.join(self.env, 'env', world_name),
                                 os.path.join(self.env, 'env', world_name))
     
+
     def env_snapshot(self, snap_name=None):
         '''snapshot [snapshot name]
         Generates a Snapshot zip file
@@ -315,6 +330,7 @@ class Server(object):
         zfile.write(self._config_file, 'baskit.config')
         zfile.close()
     
+
     def env_snap_restore(self, snap_name):
         '''snap_restore [snapshot name]
         Restores a snapshot zipfile.
@@ -365,6 +381,7 @@ class Server(object):
         else:
             return False
     
+
     def world_restore(self, backup_name, world_name):
         '''world_restore [backup name]
         Restores a world backup.  If restoring to a new world, then the
@@ -400,39 +417,62 @@ class Server(object):
             zfile = ZipFile(os.path.join(self.env, 'archive', 'backups',
                                          '%s.zip' % backup_name))
             zfile.extractall(world_path)
-            
-            # Lastly lets check to see if this world name exists in our config
-            # and create the world object if needed.
-            exists = False
-            if not any(world_name == world.name for world in self.worlds):
-                self.worlds.append(World(world_name))
-                self.set_config()
             return True
         else:
             return False
-    
-    def world_add(self, world_name):
-        '''world_add [world name]
-        Adds a new world to the running configuration.
+
+
+    def prsync(self, worlds=None):
+        '''prsync
+        Performs a sync from the persistent data to the ramdisk
         '''
-        self.worlds.append(World(world_name))
-        self.set_config()
+        if worlds == None:
+            worlds = self.worlds
+        if self.ramdisk:
+            for world in worlds:
+                self._check_path(os.path.join(self.env, 'env', world))
+                self._check_path(os.path.join(self.env, 'persistent', world))
+                self._sync(os.path.join(self.env, 'persistent', world),
+                           os.path.join(self.env, 'env', world))
     
-    def world_rm(self, world_name):
-        '''world_rm [world name]
-        Removes the specified world from the running server config.  The
-        configuration data for this world will still exist, however will be
-        removed from the list of active worlds that the server is talking to.
+
+    def rpsync(self, worlds=None):
+        '''rpsync
+        Performs a sync from the ramdisk to the persistent data
         '''
-        for world in self.worlds:
-            if world.name == world_name:
-                self.worlds.remove(world)
-        self.set_config()
+        if worlds == None:
+            worlds = self.worlds
+        if self.ramdisk:
+            for world in worlds:
+                self._check_path(os.path.join(self.env, 'env', world))
+                self._check_path(os.path.join(self.env, 'persistent', world))
+                self._sync(os.path.join(self.env, 'env', world),
+                           os.path.join(self.env, 'persistent', world))
     
-    def world_get(self, world_name):
-        '''world_get [world name]
-        Returns the world of the specified name.
+
+    def _check_path(self, path):
+        if not os.path.exists(path):
+            os.makedirs(path)
+
+    def _sync(self, from_path, to_path):
+        '''Internal Function
+        Convenience function for rsyncing.  Will use a lockfile with the
+        world name to prevent multipe syncs from occuring at the same time to
+        the same files.
         '''
-        for world in self.worlds:
-            if world_name == world.name:
-                return world
\ No newline at end of file
+        rsync_cmd = 'rsync -r -t %s/ %s/' % (from_path, to_path)
+        world = os.path.split(from_path)[1]
+        lockfile = os.path.join(self.env, 'persistent', '%s.lock' % world)
+        
+        # Wait for the lockfile to be released if one exists
+        while os.path.exists(lockfile):
+            time.sleep(0.1)
+        
+        # Create the lockfile
+        lf = open(lockfile, 'w').close()
+        
+        # Run the Rsync
+        run(rsync_cmd)
+        
+        # Remove the lockfile
+        os.remove(lockfile)
\ No newline at end of file
